%analyze_election_data
% LAST UPDATED by Andy French April 2020

function analyze_election_data
analyze_election_year(2010)
analyze_election_year(2015)
analyze_election_year(2017)
analyze_election_year(2019)

%%

function analyze_election_year(year)
global d

%fontsize for graphs
fsize = 16;

%Ignore parties with less than this percentage of total vote in plots
ignore_percent = 0.2;

%Ignore parties with less than this percentage of total vote in PR calculations
ignore_percent_PR = 1;

%Scales for total seats and maximum vote percent
max_seats = 400;
max_vote_percent = 50;

%Max radius of constituency vote star graph
max_constituency_vote = 48e3;

%Party RGB colours
%http://blog.richardallen.co.uk/uk-political-party-web-colours/
%https://nihonium.co.uk/projects/politics/political_party_colours.html
d.C_RGB = [0,135,220]/255;
d.L_RGB = [146,0,13]/255;
d.LD_RGB = [253,187,48]/255;
d.SNP_RGB = [255,249,93]/255;
d.PC_RGB = [63,132,40]/255;
d.G_RGB = [0,116,95]/255;
d.UKIP_RGB = [239,230,0]/255;
d.DUP_RGB = [212,106,76]/255;
d.SF_RGB = [0,104,55]/255;
d.Brexit_RGB = [18,182,207]/255;
d.other_RGB = [0.2,0.2,0.2];

%

%Load data
load UK_Election;
if year==2017
    e = election2017;
elseif year==2015
    e = election2015;
elseif year==2019
     e = election2019;
else
    e = election2010;
end

%Create destination direction for plots
mkdir([num2str(year),' Election']);

%

%%% % VOTE BAR CHART %%%

%Plot bar chart of % of vote, ordered by parties
[total_vote_percent,i] = sort( e.total_vote_percent,'descend' );
ignore = find( total_vote_percent < ignore_percent );
total_vote_percent(ignore) = [];
total_votes = e.total_votes(i); total_votes(ignore) = [];
party_abvs = pa(i); party_abvs(ignore) = [];

%Plot horizontally stacked bar graphs, coloured by party
for n=1:length(total_vote_percent);
    y = total_vote_percent(n);
    barh( n,y,'facecolor',party_colours( party_abvs{n} ) ); hold on;
    text( y + 0.5, n, num2str(y,2),'fontsize',fsize/1.5 );
end
set( gca,'ytick',1:length(party_abvs),'yticklabel',party_abvs,'fontsize',fsize );
set(gca,'fontsize',fsize); xlabel('Total vote %','fontsize',fsize);
xlim([0,max_vote_percent]);
title( [num2str(year),' election total vote % of ',...
    num2str( sum(e.total_votes) ),' votes.'],'fontsize',fsize );
grid on;
print( gcf,[num2str(year),' Election\',num2str(year),' total votes.png'],'-dpng','-r300' );
close(gcf);

%

%%% PARLAMENTARY SEATS VS PR BAR CHART %%%

%Determine additional seats per party to make sure total seats match
%PR distribution. Determine PR extra seat allocations and save this to a text file.
[ seats, total_vote_percent,...
    pr_seats,extra_seats,new_seat_proportions,pr_proportions,...
    extra_PR_seats, party_abvs, ignored_party_abvs, ignored_seats, s, results ] =...
    PR_calculation(e, ignore_percent_PR, pa );

%Save results to a text file
fid = fopen([num2str(year),' Election\',num2str(year),' FPTP + PR MPs.txt'],'w+');
fwrite(fid,s); fclose(fid);

%Perform PR distribution check
disp(' ');
disp( [' Check new seat to PR proportions for ',num2str(year),' election'] );
disp(' FPTP seats vs With extra PR seats vs ideal PR ' );
PR_check = {'PARTY','FPTP seats','With extra seats','PR'}; 
for n=1:length(party_abvs); 
    PR_check{n+1,1} = party_abvs{n}; 
    PR_check{n+1,2} = (seats(n)/sum(e.seats));
    PR_check{n+1,3} = new_seat_proportions(n);
    PR_check{n+1,4} = pr_proportions(n);
end
PR_check

%Save election results to a .mat file
save( [num2str(year),' Election\',num2str(year),' results.mat'],'results' );

%Determine party abbreviations and hence ticklabels
yticklabels = {}; y = [];
for n=1:length(party_abvs)
    yticklabels = [yticklabels,party_abvs{n},...
        [party_abvs{n},' PR'],[party_abvs{n},' PRextra']];
    y = [y,seats(n),pr_seats(n),extra_seats(n)];
end

%Plot horizontally stacked bar graphs, coloured by party
for n=1:length(y);
    barh( n,y(n),'facecolor',party_colours( yticklabels{n} ) ); hold on;
    text( y(n) + 3, n, num2str(y(n)),'fontsize',fsize/1.5 );
end

%Plot any ignored seats (note these won't have a PR correction)
if ~isempty(ignored_seats)
    for n=1:length(ignored_seats);
        barh( n+length(y),ignored_seats(n),'facecolor',...
            party_colours( ignored_party_abvs{n} ) ); hold on;
        text( ignored_seats(n) + 3, n+length(y), num2str( ignored_seats(n) ),'fontsize',fsize/1.5 );
    end
end

%Graph annotations
set( gca,'ytick',1:(3*length(party_abvs)+length(ignored_seats)),...
    'yticklabel',[yticklabels,ignored_party_abvs]);
set(gca,'fontsize',fsize/1.5); xlabel('Parliamentary seats','fontsize',fsize);
title( [num2str(year),' election: Seats= ',...
    num2str( sum(e.seats) ),'. PR seats=',num2str( sum(pr_seats)+sum(ignored_seats) ),...
    '. Extra PR seats=',num2str( sum(extra_seats) )],'fontsize',fsize );
grid on; box on; xlim([0,max_seats]);
print( gcf,[num2str(year),' Election\',num2str(year),' total seats.png'],'-dpng','-r300' );
close(gcf);

%

%%% VOTES PER PARTY BY CONSTITUENCY STAR PLOT %%%%

%Assemble arrays to plot
[seats,i] = sort( e.seats,'descend' );
ignore = find( e.total_vote_percent(i) < ignore_percent_PR  );
v = e.total_vote_by_constituency(:,i)/1000; v(:,ignore) = [];
mean_vote_per_constituency = mean( sum( e.total_vote_by_constituency, 2 ) )/1e3;
parties = e.parties(i); parties(ignore) = [];
party_abvs = pa(i); party_abvs(ignore) = [];

%List average votes per constituency per party
mean_party_vote_per_constituency = mean( v, 1 );
sd_party_vote_per_constituency = std( v, 1 );
disp( [ num2str(year),' constituency vote averages: '] );
for n=1:length(parties);
    disp( [ parties{n},' : ',num2str(mean_party_vote_per_constituency(n),3), ' +/- ',...
        num2str(sd_party_vote_per_constituency(n),3),' (thousand) votes.' ] );
end
disp(' ');

%Plot votes per party as a 'star graph' by constituency
N = length(e.constituencies);
theta = linspace(0,2*pi,N).'; x0 = 0;
for n=1:length(parties)
    %x offset
    x0 = x0 + max(v(:,n));
    if n==1
        %Plot black circle to indicate scale
        t = linspace(0,2*pi,1000);
        x = max_constituency_vote*cos(t)/1000;
        y = max_constituency_vote*sin(t)/1000;
        plot(x + x0,y,'k','HandleVisibility','off'); hold on;
    end
    x = v(:,n).*cos(theta) + x0;
    y =  v(:,n).*sin(theta);
    p = patch(x,y,party_colours( party_abvs{n} )); hold on;
    set( p, 'edgecolor',party_colours( party_abvs{n} ) );
    x0 = x0 + max( v(:,n) );
end
axis equal; xlims = get(gca,'xlim'); set(gca,'xlim',[xlims(1)-1,xlims(2)] );
set(gca,'fontsize',fsize); 
legend(party_abvs,'fontsize',fsize,'orientation','horizontal',...
    'location','southoutside');
title( [num2str(year),' votes by constituency'],'fontsize',fsize )
axis off;
print( gcf,[num2str(year),' Election\',num2str(year),' contituency votes.png'],'-dpng','-r300' );
close(gcf);

%

%%% VOTES PER PARTY BY CONSTITUENCY STACKED BAR GRAPH %%%%
ba = bar(v,'stacked'); 
for n=1:length(party_abvs)
    set( ba(n),'Facecolor',party_colours( party_abvs{n} ),...
        'Edgecolor', party_colours( party_abvs{n} ) );
end
set(gca,'fontsize',fsize); axis tight; ylim([0,80]); grid on; box on;
xlabel('Constituency number','fontsize',fsize)
ylabel('Votes per constituency /1000','fontsize',fsize );
title([num2str(year),' votes per constituency /1000. Mean=',...
    num2str(mean_vote_per_constituency,3)],'fontsize',fsize );
legend(party_abvs,'fontsize',fsize,'orientation','vertical',...
    'location','eastoutside');
print( gcf,[num2str(year),' Election\',num2str(year),' contituency votes bargraph.png'],'-dpng','-r300' );
close(gcf);

%

%%% VOTES PER PARTY BY REGION STACKED BAR GRAPH %%%%

%Determine data vectors to plot
[seats,i] = sort( e.seats,'descend' );
ignore = find( e.total_vote_percent(i) < ignore_percent  );
parties = e.parties(i); parties(ignore) = [];
party_abvs = pa(i); party_abvs(ignore) = [];
N = length(e.regions);
v = e.total_vote_by_region(:,i); v(:,ignore) = [];

%List average votes per region per party
mean_party_vote_per_region = mean( v, 1 )/1e6;
sd_party_vote_per_region = std( v, 1 )/1e6;
disp( [ num2str(year),' regional vote averages: '] );
for n=1:length(parties);
    disp( [ parties{n},' : ',num2str(mean_party_vote_per_region(n),3), ' +/- ',...
        num2str(sd_party_vote_per_region(n),3),' (million) votes.' ] );
end
disp(' ');

%Construct bar graph
ba = barh(v/1e6,'stacked'); 
for n=1:length(party_abvs)
    set( ba(n),'Facecolor',party_colours( party_abvs{n} ),...
        'Edgecolor', party_colours( party_abvs{n} ) );
end
hold on; dim = size(v); plot( sum(v,2)/1e6, (1:dim(1)).','k','linewidth',2 );
set(gca,'fontsize',fsize); axis tight; xlim([0,5]); grid on;
set( gca, 'yticklabel',regions,'fontsize',fsize/1.5)
xlabel('Votes per region (millions)','fontsize',fsize );
title([num2str(year),' votes per region (millions). Total=',num2str(sum(e.total_votes)/1e6,4)],'fontsize',fsize );
legend(party_abvs,'fontsize',fsize,'orientation','vertical',...
    'location','eastoutside');
print( gcf,[num2str(year),' Election\',num2str(year),' region votes bargraph.png'],'-dpng','-r300' );
close(gcf);

%

%%% VOTES PER PARTY CIRCLES BY REGION %%%

%Plot votes per party as a blob graph by region

%Load UK regional map
I = imread('UK regions blank.png'); s = size(I); xpix = s(2); ypix = s(1);

%Step through parties, and overlay circles on a UK map to represent votes
theta = linspace(0,2*pi,500); II = [];
for n=1:length(parties)
    image(I); axis equal; hold on;
    axis equal; axis tight; axis off;
    for m=1:N
        xy = regionxy( regions{m} );
        r = sqrt( v(m,n)/(pi*100) );
        x = r*cos(theta) + xy(1);
        y =  r*sin(theta) + ypix - xy(2);
        p = patch(x,y,party_colours( party_abvs{n} ));
        set( p, 'edgecolor',party_colours( party_abvs{n} ) );
    end
    set(gca,'fontsize',fsize);
    title(party_abvs{n},'fontsize',fsize);
    
    %Print PNG image, import it, trim it, and add to bitmap array II
    print(gcf,'temp.png','-dpng','-r300'); temp = imread('temp.png');
    II = [II,temp(:,500:1900,:)]; clf;
end
close(gcf); delete('temp.png')

%Write combined bitmap for all parties
imwrite( II, [num2str(year),' Election\',num2str(year),' regional votes.png'] );

%

%%% VOTES PER PARTY CONSTITUENCY STAR PLOTS BY REGION %%%

%Plot votes per party as a constituency star by region
[seats,i] = sort( e.seats,'descend' );
ignore = find( e.total_vote_percent(i) < ignore_percent  );
parties = e.parties(i); parties(ignore) = [];
party_abvs = pa(i); party_abvs(ignore) = [];
N = length(e.regions);

%Load UK regional map
I = imread('UK regions blank.png'); s = size(I); xpix = s(2); ypix = s(1);

%Step through parties, and overlay circles on a UK map to represent votes
theta = linspace(0,2*pi,500); II = [];
for n=1:length(parties)
    image(I); axis equal; hold on;
    axis equal; axis tight; axis off;
    for m=1:N
        xy = regionxy( regions{m} );
        theta = linspace(0,2*pi,length( e.regions(m).constituencies ) );
        v = e.regions(m).votes_by_constituency(:,i); v(:,ignore) = [];
        v = v(:,n).'/400;
        x = v.*cos(theta) + xy(1);
        y =  v.*sin(theta) + ypix - xy(2);
        p = patch(x,y,party_colours( party_abvs{n} ));
        set( p, 'edgecolor',party_colours( party_abvs{n} ) );
    end
    set(gca,'fontsize',fsize);
    title(party_abvs{n},'fontsize',fsize);
    
    %Print PNG image, import it, trim it, and add to bitmap array II
    print(gcf,'temp.png','-dpng','-r300'); temp = imread('temp.png');
    II = [II,temp(:,500:1900,:)]; clf;
end
close(gcf); delete('temp.png')

%Write combined bitmap for all parties
imwrite( II, [num2str(year),' Election\',num2str(year),' regional constituency votes.png'] );

%%

%Determine party colours from abbreviation string abv
function RGB = party_colours(abv)
global d
a = '      ';
if length(abv)<=6; a( 1:length(abv) ) = abv; else a = abv(1:6); end
if strcmp(a(1),'C'); RGB = d.C_RGB;
elseif strcmp(a(1:3),'Lab'); RGB = d.L_RGB;
elseif strcmp(a(1:2),'LD'); RGB = d.LD_RGB;
elseif strcmp(a(1:3),'SNP'); RGB = d.SNP_RGB;
elseif strcmp(a(1:2),'PC'); RGB = d.PC_RGB;
elseif strcmp(a(1:5),'Green'); RGB = d.G_RGB;
elseif strcmp(a(1:4),'UKIP'); RGB = d.UKIP_RGB;
elseif strcmp(a(1:2),'SF'); RGB = d.SF_RGB;
elseif strcmp(a(1:3),'DUP'); RGB = d.DUP_RGB;
elseif strcmp(a(1:6),'Brexit'); RGB = d.Brexit_RGB;
else RGB = d.other_RGB;
end

%%

%Determine region x,y coordinates from region string
function xy = regionxy(region)

%UK Regional x,y coordinates (pixels from bottom left of UK regions
%blank.png)
xy_E = [832,305];
xy_E_Mid = [713,366];
xy_London = [770,200];
xy_NorthE = [621,616];
xy_NorthW = [569,502];
xy_NI = [304,565];
xy_Scot = [466,809];
xy_SE = [709,117];
xy_SW = [504,109];
xy_Wales = [494,272];
xy_W_Mid = [605,311];
xy_Yorks = [678,495];
if strcmp( region, 'East' )==1
    xy = xy_E;
elseif strcmp( region, 'East Midlands' )==1
    xy = xy_E_Mid;
elseif strcmp( region, 'London' )==1
    xy = xy_London;
elseif strcmp( region, 'North East' )==1
    xy = xy_NorthE;
elseif strcmp( region, 'North West' )==1
    xy = xy_NorthW;
elseif strcmp( region, 'Northern Ireland' )==1
    xy = xy_NI;
elseif strcmp( region, 'Scotland' )==1
    xy = xy_Scot;
elseif strcmp( region, 'South East' )==1
    xy = xy_SE;
elseif strcmp( region, 'South West' )==1
    xy = xy_SW;
elseif strcmp( region, 'Wales' )==1
    xy = xy_Wales;
elseif strcmp( region, 'West Midlands' )==1
    xy = xy_W_Mid;
elseif strcmp( region, 'Yorkshire and The Humber' )==1
    xy = xy_Yorks;
end

%%

%Calculate extra seats to make proportional representation work
%Each element of arrays correspond to parties. Seats are actual
%first-past-the-post constituency seats won.
function [pr_seats,extra_seats,new_seat_proportions,pr_proportions] =...
    calc_extra_pr_seats( total_vote_percent, seats )

%Calculate the seats that should have been won, if the number of seats
%is in proportion to the total national vote
pr_seats = round(total_vote_percent*sum(seats)/sum(total_vote_percent));

%Determine maximum ratio between seats and PR seats.
rmax = max( seats./pr_seats );

%Calculate extra seats
extra_seats = round( pr_seats*rmax - seats );

%Compute seat proportions and compare to PR ideal
new_seat_proportions = (extra_seats + seats)/sum( extra_seats + seats );
pr_proportions = pr_seats/sum(pr_seats);

%%

%Determine which constituencies get extra MPs to make PR work
%extra_PR_seats is an array of seats, with rows corresponding to constituencies
%and cols corresponding to parties. s is the string version to be displayed
%in the command window or written to a file.
function [ seats, total_vote_percent,...
    pr_seats,extra_seats,new_seat_proportions,pr_proportions,...
    extra_PR_seats, party_abvs, ignored_party_abvs, ignored_seats, s, results ] =...
    PR_calculation(e, ignore_percent, pa )

%Sort total_vote_percent
[total_vote_percent,isort] = sort( e.total_vote_percent,'descend' );

%Any ignored parties that actually have seats?
ignored_seats = e.seats(isort);
i = find( (ignored_seats >0) & (total_vote_percent < ignore_percent) );
ignored_seats = ignored_seats(i);
ignored_party_abvs = pa(isort).';
ignored_party_abvs = ignored_party_abvs(i);

%Remove parties with less than the ignore_percent of vote
seats = e.seats(isort);
ignore = find( total_vote_percent < ignore_percent );
seats(ignore) = []; total_vote_percent(ignore) = [];

%Determine additional seats per party to make sure total seats match
%PR distribution
[pr_seats,extra_seats,new_seat_proportions,pr_proportions] =...
    calc_extra_pr_seats( total_vote_percent, seats );

%Determine party abbreviations
party_abvs = pa(isort); party_abvs(ignore) = [];
parties = e.parties(isort); parties(ignore) = [];

%Remove winning party, (and any parties not in cell array parties) from
%total_vote_by_constituency). Do this by setting the number of votes
%per constituency of the winning party to be -1.
v = e.total_vote_by_constituency(:,isort); v(:,ignore) = [];
[Nc,Np] = size(v);
for n=1:Nc
    i = find( v(n,:)==max( v(n,:) ) ); i = i(1);
    winner(n) = i;
    v( n,winner(n) ) = -1;
end

%For each party, sort votes by constituency in descending order, and hence
%allocate extra seats to the most appropriate constituencies. Do in order
%of most popular party to minimize strange results which can result from
%insufficient constituencies to find suitable extra MPs from.
extra_PR_seats = zeros(Nc,Np);
for nn=1:Np
    %Sort votes per constituency for party n
    n = Np-nn+1;
    vn = v(:,n);
    [vn,j] = sort( vn, 'descend' );
    
    %Allocate extra seats
    for m=1:extra_seats(n)
        extra_PR_seats( j(m), n ) = 1;
    end
    
    %Prevent more than one extra MP per constituency, by forcing the
    %constituencies with the extra seats to be bottom of the ordered list
    v( j( 1:extra_seats(n) ),: ) = -1;
end

%Determine text string of results, and store results in structure results
s = '';
for n=1:Nc
    
    %Add results to structure results
    results(n).constituency = e.constituencies(n).constituency;
    v = e.constituencies(n).votes; [v,i] = sort( v, 'descend' );
    vp = e.constituencies(n).vote_percent(i);
    p = e.constituencies(n).parties(i);
    ignore = find( vp < ignore_percent ); v(ignore) = [];
    vp(ignore) = []; p(ignore) = [];
    results(n).parties = p;
    results(n).votes = v;
    results(n).vote_percent = vp;
    
    %Add to text string of results (which will be written to a file)
    spaces = max( [1, 44 - length( e.constituencies(n).constituency ) ] );
    spaces = repmat( ' ',[1,spaces] );
    ss = ['FPTP MP=',party_abvs{winner(n)} ];
    for m=1:length(party_abvs)
        if extra_PR_seats(n,m)==1
            
            %Determine position in constituency vote of added part
            pos = find( strcmp( e.constituencies(n).parties(i), parties{m})==1 );
            if pos>3
                %Result is untennable, so remove from list
                extra_PR_seats(n,m)=0;
                extra_seats(m) = extra_seats(m) - 1 ;
            elseif pos==2
                %OK with second place being an extra MP
                ss = [ss,', Extra MP=',party_abvs{m},' (',num2str(pos),')'];
            else
                %Somewhat undesirable, so indicate with a *
                ss = [ss,', Extra MP=',party_abvs{m},' (',num2str(pos),') **'];
            end
        end
    end
    results(n).MPs = ss;
    s = [s,e.constituencies(n).constituency,...
        ':',spaces,ss,char(10)];
end

%Compute proportions check
new_seat_proportions = sum( extra_PR_seats, 1 ) + seats;
new_seat_proportions = new_seat_proportions/sum(new_seat_proportions);

%End of code