%get_UK_election_data_2010_to_2019
% Loads spreadsheet of UK election data and creates various data structures
% to enable analysis by year party, constituency and region
%
% The resulting data structure is then saved to a .mat file UK_Election
%
% LAST UPDATED by Andy French Nov 2019
%
% https://data.gov.uk/dataset/b77fcedb-4792-4de4-935f-4f344ed4c2c6/general-election-results-2017
% https://researchbriefings.parliament.uk/ResearchBriefing/Summary/CBP-8749

function get_UK_election_data_2010_to_2019

%Display message to command window and start clock
t0=tic;
disp(' ');
disp(' **** Getting UK 2010-2019 election data **** ')
disp(' ');

%Load UK Election data
disp(' ... Importing 2010-2017 Excel data from data.gov.uk');
[num,txt,raw] = xlsread('2010-2017 General Election Results'); [rows,cols] = size(raw);

%Determine all the possible parties, regions, and all contituencies
constituencies = unique( raw(2:end,3) ); regions = unique( raw(2:end,4) );
[parties,i,j] = unique( raw(2:end,6)); pa = raw(2:end,5); pa = pa(i);
[pa,i,j] = unique( [pa;'Brexit';'Other'] ); parties = [parties;'Brexit';'Other']; parties = parties(i);

%Step through data, and for each region, determine the constituencies
for n=1:length(regions)
    region_constituencies(n).region = regions{n};
    region_constituencies(n).constituencies = {};
    for k=2:rows
        if strcmp( raw(k,4),regions{n} )==1
            region_constituencies(n).constituencies =...
                [region_constituencies(n).constituencies,...
                raw(k,3) ];
        end
    end
    region_constituencies(n).constituencies =...
        unique( region_constituencies(n).constituencies );
end

%Get data for each year
election2010 = get_data_by_year( 2010,raw, constituencies, regions,...
    parties, region_constituencies );
election2015 = get_data_by_year( 2015,raw, constituencies, regions,...
    parties, region_constituencies );
election2017 = get_data_by_year( 2017,raw, constituencies, regions,...
    parties, region_constituencies );

%2019 data has a different Excel source file
disp(' ... Importing 2019 Excel data');
[num,txt,raw] = xlsread('2019 General Election Results.xlsx'); [rows,cols] = size(raw);
election2019 = get_data_by_year( 2019,raw, constituencies, regions,...
    parties, region_constituencies );

%Save data
disp(' ... Saving .mat file')
save UK_Election parties pa regions region_constituencies constituencies election2017 election2015 election2010 election2019
disp( [' ... Election data structure saved in ',num2str( toc(t0) ),'s.'] );
disp(' ');

%%

%Get data by year
function e = get_data_by_year( year,raw,constituencies,regions,parties,region_constituencies )
[rows,cols] = size(raw);

%Display message to command window
disp( [' ... Structuring data for ',num2str(year), ' election.'] );

%Initialize data structure fields with e, which corresponds to the election
%results for year 'year'

%Total parliamentary seats by party
e.parties = parties;
e.seats = zeros(1,length(parties));

%Total votes
e.total_votes = zeros(1,length(parties));
e.total_vote_percent = zeros(1,length(parties));
e.total_vote_by_region = zeros( length(regions),length(parties) );
e.total_vote_percent_by_region = zeros( length(regions),length(parties) );
e.total_vote_by_constituency = zeros( length(constituencies),length(parties) );
e.total_vote_percent_by_constituency = zeros( length(constituencies),length(parties) );

%Votes within each region
for n=1:length(regions)
    RC = length( region_constituencies(n).constituencies );
    e.regions(n).region = regions{n};
    e.regions(n).parties = parties;
    e.regions(n).votes = zeros(1,length(parties));
    e.regions(n).vote_percent = zeros(1,length(parties));
    e.regions(n).votes_by_constituency = zeros(RC,length(parties));
    e.regions(n).vote_percent_by_constituency = zeros(RC,length(parties));
    for m=1:RC
        e.regions(n).constituencies(m).constituency = region_constituencies(n).constituencies{m};
        e.regions(n).constituencies(m).parties = parties;
        e.regions(n).constituencies(m).votes = zeros(1,length(parties));
        e.regions(n).constituencies(m).vote_percent = zeros(1,length(parties));
    end
end

%Votes by constituency, ignoring regional classification
for n=1:length(constituencies)
    e.constituencies(n).constituency = constituencies{n};
    e.constituencies(n).parties = parties;
    e.constituencies(n).votes = zeros(1,length(parties));
    e.constituencies(n).vote_percent = zeros(1,length(parties));
end

%

%Step through Excel sheet and add to data structures
if year~=2019
    for n=2:rows
        y = raw{n,1}; c =raw{n,3}; r = raw{n,4};
        party = raw{n,6}; votes = raw{n,7};
        pi = strcmp( parties, party ); pi = find( pi==1 );
        ci = strcmp( constituencies, c ); ci = find( ci==1 );
        ri = strcmp( regions, r ); ri = find( ri==1 );
        rci = strcmp( region_constituencies(ri).constituencies, c ); rci = find( rci==1 );
        if y==year
            e.total_votes(pi) = e.total_votes(pi) + votes;
            e.total_vote_by_region(ri,pi) = e.total_vote_by_region(ri,pi) + votes;
            e.total_vote_by_constituency(ci,pi) = votes;
            e.regions(ri).votes_by_constituency(rci,pi) = votes;
            e.regions(ri).votes(pi) = e.regions(ri).votes(pi) + votes;
            e.constituencies(ci).votes(pi) = votes;
            e.regions(ri).constituencies(rci).votes(pi) = votes;
        end
    end
end

if year==2019
    %Step through Excel sheet and add to data structures
    for n=2:rows
        for cols=19:31
            c =raw{n,3}; r = raw{n,5};
            party = raw{1,cols}; votes = raw{n,cols};
            pi = strcmp( parties, party ); pi = find( pi==1 );
            ci = strcmp( constituencies, c ); ci = find( ci==1 );
            ri = strcmp( regions, r ); ri = find( ri==1 );
            rci = strcmp( region_constituencies(ri).constituencies, c ); rci = find( rci==1 );
            e.total_votes(pi) = e.total_votes(pi) + votes;
            e.total_vote_by_region(ri,pi) = e.total_vote_by_region(ri,pi) + votes;
            e.total_vote_by_constituency(ci,pi) = votes;
            e.regions(ri).votes_by_constituency(rci,pi) = votes;
            e.regions(ri).votes(pi) = e.regions(ri).votes(pi) + votes;
            e.constituencies(ci).votes(pi) = votes;
            e.regions(ri).constituencies(rci).votes(pi) = votes;
        end
    end
end

%

%Now determine percent of vote

%Total votes
e.total_vote_percent = 100*e.total_votes/sum( e.total_votes);

%Total votes by region
for n=1:length(regions)
    e.total_vote_percent_by_region(n,:) =...
        100*e.total_vote_by_region(n,:)/sum( e.total_vote_by_region(n,:));
end

%Total votes by constituency
for n=1:length(constituencies)
    e.total_vote_percent_by_constituency(n,:) =...
        100*e.total_vote_by_constituency(n,:)/sum( e.total_vote_by_constituency(n,:));
    e.constituencies(n).vote_percent =...
        100*e.constituencies(n).votes/sum( e.constituencies(n).votes);
end

%Determine number of parliametary seats, based upon first-past-the-post
%system
for n=1:length(constituencies)
    winner = find( e.constituencies(n).vote_percent ==...
        max( e.constituencies(n).vote_percent ) );
    e.seats(winner) = e.seats(winner) + 1;
end

%Votes within each region, and then within each constituency per region
for n=1:length(regions)
    e.regions(n).vote_percent =...
        100*e.regions(n).votes/sum( e.regions(n).votes);
    for m=1:length( e.regions(n).constituencies )
        e.regions(n).constituencies(m).vote_percent =...
            100*e.regions(n).constituencies(m).votes/sum( e.regions(n).constituencies(m).votes);
        e.regions(n).vote_percent_by_constituency(m,:) =...
            100*e.regions(n).votes_by_constituency(m,:)/sum( e.regions(n).votes_by_constituency(m,:));
    end
end

%End of code
